# ./addons/eventscripts/enemyradar/enemyradar.py

import es
import gamethread
import vecmath
from playerlib import getPlayer
from usermsg import hudhint

class EnemyRadar(object):
    class Player(object):
        def __init__(self, userid):
            self.location  = None
            self.userid    = userid
            self.player    = getPlayer(userid)

        def playerSpawn(self):
            x, y, z       = es.getplayerlocation(self.userid)
            self.location = vecmath.vector((float(x), float(y), float(z)))
            gamethread.delayed(.1, hudhint, (self.userid, '[Enemy-Radar] Initialising...'))
            gamethread.delayedname(1.5, 'enemyradar_%s' % self.userid, self._distanceMeter)

        def cancel(self):
            hudhint(self.userid, '[Enemy-Radar] Offline.')
            gamethread.cancelDelayed('enemyradar_%s' % self.userid)

        def _distanceMeter(self):
            distances = []
            near      = self.player.getNearPlayers(2000)
            if near:
                for userid in filter(lambda x: es.getplayerteam(x) != es.getplayerteam(self.userid), near):
                    distances.append('%s (%s meters), ' % (es.getplayername(userid), round(vecmath.distance(self.location, vecmath.vector(es.getplayerlocation(userid))) * 0.0254, 2)))
                if distances:
                    es.emitsound('player', self.userid, 'enemyradar/sonar1.wav', 1.0, 1.0)
                    hudhint(self.userid, '[Enemy-Radar]\n\n%s' % ''.join(distances).replace(', ', '\n'))
                else:
                    hudhint(self.userid, '[Enemy-Radar]\n\nNo players in range of 2000 units.')
            else:
                hudhint(self.userid, '[Enemy-Radar]\n\nNo players in range of 2000 units.')
            gamethread.delayedname(2, 'enemyradar_%s' % self.userid, self._distanceMeter)

    def __init__(self):
        self.players = {}

    def scriptUnload(self):
        for userid in self.players:
            self.players[userid].cancel()
        self.players.clear()

    def doDownloads(self):
        es.stringtable('downloadables', 'sound/enemyradar/sonar1.wav')

    def playerSpawn(self, userid, startSpawn):
        if startSpawn:
            if userid not in self.players:
                self.players[userid] = self.Player(userid)
            self.players[userid].playerSpawn()

    def playerDeath(self, userid):
        if userid in self.players:
            self.players[userid].cancel()

    def playerDisconnect(self, userid):
        if userid in self.players:
            self.players[userid].cancel()
            del self.players[userid]

    def roundEnd(self):
        for userid in self.players:
            self.players[userid].cancel()
er = EnemyRadar()

def es_map_start(ev):
    er.doDownloads()

def round_end(ev):
    er.roundEnd()

def player_spawn(ev):
    er.playerSpawn(ev['userid'], int(ev['es_userteam'] > 1 and ev['es_steamid'] != 'BOT'))

def player_death(ev):
    er.playerDeath(ev['userid'])

def player_disconnect(ev):
    er.playerDisconnect(ev['userid'])

def load():
    er.doDownloads()

def unload():
    er.scriptUnload()
